package simpleex.json;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.module.SimpleModule;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import simpleex.core.LatLong;
import simpleex.core.LatLongs;

public class LatLongsJsonTest extends AbstractJsonTest {

  @Before
  @Override
  public void setUp() {
    super.setUp();
  }

  @Override
  protected ObjectMapper createObjectMapper() {
    final SimpleModule module =
        createSimpleModule(LatLong.class, new LatLongSerializer(), new LatLongDeserializer());
    module.addSerializer(LatLongs.class, new LatLongsSerializer());
    module.addDeserializer(LatLongs.class, new LatLongsDeserializer());
    return new ObjectMapper().registerModule(module);
  }

  @Test
  public void testLatLongsSerialization() throws Exception {
    assertWriteValue(
        "[{\"latitude\":63.1,\"longitude\":12.3},{\"latitude\":63.0,\"longitude\":12.4}]",
        new LatLongs(new LatLong(63.1, 12.3), new LatLong(63.0, 12.4)));
  }

  @Test
  public void testLatLongsDeserialization() throws Exception {
    final LatLongs latLongs =
        readValue("[{\"latitude\":63.1,\"longitude\":12.3}, [63.0,12.4]]", LatLongs.class);
    Assert.assertEquals(2, latLongs.getLatLongCount());
    Assert.assertEquals(new LatLong(63.1, 12.3), latLongs.getLatLong(0));
    Assert.assertEquals(new LatLong(63.0, 12.4), latLongs.getLatLong(1));
  }
}
